@@ -7,6 +7,7 @@ ac_table = {
         'executor',
         'flow',
         'ping',
+        'new',
         'gateway',
         'hub',
         'help',
@@ -32,7 +33,7 @@ ac_table = {
             '--workdir',
             '--download-proxy',
             '--index-data-url',
-            '--port-expose',
+            '--port',
             '--replicas',
             '--unblock-query-flow',
         ],
@@ -41,7 +42,7 @@ ac_table = {
             '--workdir',
             '--download-proxy',
             '--index-data-url',
-            '--port-expose',
+            '--port',
             '--unblock-query-flow',
         ],
         'hello fork': ['--help', 'fashion', 'chatbot', 'multimodal'],
@@ -67,6 +68,7 @@ ac_table = {
             '--port-in',
             '--host-in',
             '--native',
+            '--output-array-type',
             '--entrypoint',
             '--docker-kwargs',
             '--pull-latest',
@@ -76,26 +78,24 @@ ac_table = {
             '--port-jinad',
             '--quiet-remote-logs',
             '--upload-files',
-            '--disable-remote',
-            '--daemon',
             '--runtime-backend',
             '--runtime',
             '--runtime-cls',
             '--timeout-ready',
             '--env',
-            '--expose-public',
             '--shard-id',
-            '--replica-id',
             '--pod-role',
             '--noblock-on-start',
             '--shards',
             '--replicas',
+            '--port',
             '--install-requirements',
             '--force-update',
             '--force',
             '--uses-before-address',
             '--uses-after-address',
             '--connection-list',
+            '--disable-reduce',
         ],
         'flow': [
             '--help',
@@ -110,11 +110,13 @@ ac_table = {
             '--k8s-namespace',
             '--k8s-disable-connection-pool',
             '--polling',
+            '--expose-graphql-endpoint',
             '--uses',
             '--env',
             '--inspect',
         ],
         'ping': ['--help', '--timeout', '--retries'],
+        'new': ['--help'],
         'gateway': [
             '--help',
             '--name',
@@ -136,6 +138,7 @@ ac_table = {
             '--port-in',
             '--host-in',
             '--native',
+            '--output-array-type',
             '--prefetch',
             '--title',
             '--description',
@@ -145,31 +148,29 @@ ac_table = {
             '--no-crud-endpoints',
             '--expose-endpoints',
             '--uvicorn-kwargs',
-            '--compress',
-            '--compress-min-bytes',
-            '--compress-min-ratio',
+            '--expose-graphql-endpoint',
             '--protocol',
             '--host',
             '--proxy',
             '--port-expose',
             '--graph-description',
+            '--graph-conditions',
             '--deployments-addresses',
-            '--daemon',
             '--runtime-backend',
             '--runtime',
             '--runtime-cls',
             '--timeout-ready',
             '--env',
-            '--expose-public',
             '--shard-id',
-            '--replica-id',
             '--pod-role',
             '--noblock-on-start',
             '--shards',
             '--replicas',
+            '--port',
             '--uses-before-address',
             '--uses-after-address',
             '--connection-list',
+            '--disable-reduce',
         ],
         'hub new': [
             '--help',
@@ -223,6 +224,7 @@ ac_table = {
             '--port-in',
             '--host-in',
             '--native',
+            '--output-array-type',
             '--entrypoint',
             '--docker-kwargs',
             '--pull-latest',
@@ -232,26 +234,24 @@ ac_table = {
             '--port-jinad',
             '--quiet-remote-logs',
             '--upload-files',
-            '--disable-remote',
-            '--daemon',
             '--runtime-backend',
             '--runtime',
             '--runtime-cls',
             '--timeout-ready',
             '--env',
-            '--expose-public',
             '--shard-id',
-            '--replica-id',
             '--pod-role',
             '--noblock-on-start',
             '--shards',
             '--replicas',
+            '--port',
             '--install-requirements',
             '--force-update',
             '--force',
             '--uses-before-address',
             '--uses-after-address',
             '--connection-list',
+            '--disable-reduce',
         ],
         'deployment': [
             '--help',
@@ -274,6 +274,7 @@ ac_table = {
             '--port-in',
             '--host-in',
             '--native',
+            '--output-array-type',
             '--entrypoint',
             '--docker-kwargs',
             '--pull-latest',
@@ -283,28 +284,27 @@ ac_table = {
             '--port-jinad',
             '--quiet-remote-logs',
             '--upload-files',
-            '--disable-remote',
-            '--daemon',
             '--runtime-backend',
             '--runtime',
             '--runtime-cls',
             '--timeout-ready',
             '--env',
-            '--expose-public',
             '--shard-id',
-            '--replica-id',
             '--pod-role',
             '--noblock-on-start',
             '--shards',
             '--replicas',
+            '--port',
             '--install-requirements',
             '--force-update',
             '--force',
             '--uses-before-address',
             '--uses-after-address',
             '--connection-list',
+            '--disable-reduce',
             '--uses-before',
             '--uses-after',
+            '--when',
             '--external',
             '--deployment-role',
         ],
@@ -315,7 +315,7 @@ ac_table = {
             '--port',
             '--https',
             '--asyncio',
-            '--results-as-docarray',
+            '--return-responses',
             '--protocol',
         ],
         'export-api': ['--help', '--yaml-path', '--json-path', '--schema-path'],
