@@ -16,10 +16,7 @@ To use these enums in YAML config, following the example below:
         parallel_type: any
 """
 
-__copyright__ = "Copyright (c) 2020 Jina AI Limited. All rights reserved."
-__license__ = "Apache-2.0"
-
-from enum import IntEnum, EnumMeta
+from enum import Enum, IntEnum, EnumMeta
 
 
 class EnumType(EnumMeta):
@@ -44,10 +41,10 @@ class EnumType(EnumMeta):
         :return: Registered class.
         """
         reg_cls_set = getattr(cls, '_registered_class', set())
-        if cls.__name__ not in reg_cls_set or getattr(cls, 'force_register', False):
+        if cls.__name__ not in reg_cls_set:
             reg_cls_set.add(cls.__name__)
             setattr(cls, '_registered_class', reg_cls_set)
-        from .jaml import JAML
+        from jina.jaml import JAML
 
         JAML.register(cls)
         return cls
@@ -103,15 +100,8 @@ class BetterEnum(IntEnum, metaclass=EnumType):
         return cls.from_string(node.value)
 
 
-class SchedulerType(BetterEnum):
-    """The enum for Scheduler Type."""
-
-    LOAD_BALANCE = 0  #: balance the workload between Peas, faster peas get more work
-    ROUND_ROBIN = 1  # : workload are scheduled round-robin manner to the peas, assuming all peas have uniform processing speed.
-
-
 class PollingType(BetterEnum):
-    """The enum for representing the parallel type of peas in a pod."""
+    """The enum for representing the parallel type of pods in a deployment."""
 
     ANY = 1  #: one of the shards will receive the message
     ALL = 2  #: all shards will receive the message, blocked until all done with the message
@@ -147,104 +137,40 @@ class LogVerbosity(BetterEnum):
     CRITICAL = 50
 
 
-class SocketType(BetterEnum):
-    """Enums for representing the socket type in a pod."""
-
-    PULL_BIND = 0
-    PULL_CONNECT = 1
-    PUSH_BIND = 2
-    PUSH_CONNECT = 3
-    SUB_BIND = 4
-    SUB_CONNECT = 5
-    PUB_BIND = 6
-    PUB_CONNECT = 7
-    PAIR_BIND = 8
-    PAIR_CONNECT = 9
-    ROUTER_BIND = 10
-    DEALER_CONNECT = 11
-
-    @property
-    def is_bind(self) -> bool:
-        """
-        Check if this socket is using `bind` protocol.
-
-        :return: True if this socket is using `bind` protocol else False.
-        """
-        return self.value % 2 == 0
-
-    @property
-    def is_receive(self) -> bool:
-        """
-        Check if this socket is used for receiving data.
-
-        :return: True if this socket is used for receiving data else False.
-        """
-        return self.value in {0, 1, 4, 5}
-
-    @property
-    def is_pubsub(self):
-        """
-        Check if this socket is used for publish or subscribe data.
-
-        :return: True if this socket is used for publish or subscribe data else False.
-        """
-        return 4 <= self.value <= 7
-
-    @property
-    def paired(self) -> 'SocketType':
-        """
-        Get the paired SocketType.
-
-        :return: a paired SocketType.
-        """
-        return {
-            SocketType.PULL_BIND: SocketType.PUSH_CONNECT,
-            SocketType.PULL_CONNECT: SocketType.PUSH_BIND,
-            SocketType.SUB_BIND: SocketType.PUB_CONNECT,
-            SocketType.SUB_CONNECT: SocketType.PUB_BIND,
-            SocketType.PAIR_BIND: SocketType.PAIR_CONNECT,
-            SocketType.PUSH_CONNECT: SocketType.PULL_BIND,
-            SocketType.PUSH_BIND: SocketType.PULL_CONNECT,
-            SocketType.PUB_CONNECT: SocketType.SUB_BIND,
-            SocketType.PUB_BIND: SocketType.SUB_CONNECT,
-            SocketType.PAIR_CONNECT: SocketType.PAIR_BIND,
-        }[self]
-
-
-class FlowOutputType(BetterEnum):
-    """The enum for representing flow output config."""
-
-    SHELL_PROC = 0  #: a shell-script, run each microservice as a process
-    SHELL_DOCKER = 1  #: a shell-script, run each microservice as a container
-    DOCKER_SWARM = 2  #: a docker-swarm YAML config
-    K8S = 3  #: a Kubernetes YAML config
-
-
 class FlowBuildLevel(BetterEnum):
     """
     The enum for representing a flow's build level.
 
-    Some :class:`jina.flow.Flow` class functions require certain build level to run.
+    Some :class:`jina.orchestrate.flow.Flow` class functions require certain build level to run.
     """
 
     EMPTY = 0  #: Nothing is built
     GRAPH = 1  #: The underlying graph is built, you may visualize the flow
+    RUNNING = 2  #: the graph is started and all deployment are running
 
 
-class PeaRoleType(BetterEnum):
-    """The enum of a Pea role."""
+class GatewayProtocolType(BetterEnum):
+    """
+    Gateway communication protocol
+    """
 
-    SINGLETON = 0
-    HEAD = 1
-    TAIL = 2
-    PARALLEL = 3
-    REPLICA = 4
+    GRPC = 0
+    HTTP = 1
+    WEBSOCKET = 2
 
 
 class PodRoleType(BetterEnum):
-    """The enum of a Pod role for visualization."""
+    """The enum of a Pod role."""
 
-    POD = 0
+    HEAD = 0
+    WORKER = 1
+    GATEWAY = 2
+
+
+class DeploymentRoleType(BetterEnum):
+    """The enum of a Deploymen role for visualization."""
+
+    DEPLOYMENT = 0
     JOIN = 1
     INSPECT = 2
     GATEWAY = 3
@@ -254,9 +180,9 @@ class PodRoleType(BetterEnum):
     @property
     def is_inspect(self) -> bool:
         """
-        If the role is inspect pod related.
+        If the role is inspect deployment related.
 
-        :return: True if the Pod role is inspect related else False.
+        :return: True if the Deployment role is inspect related else False.
         """
         return self.value in {2, 4}
 
@@ -264,14 +190,8 @@ class PodRoleType(BetterEnum):
 class RequestType(BetterEnum):
     """The enum of Client mode."""
 
-    INDEX = 0
-    SEARCH = 1
-    DELETE = 2
-    UPDATE = 3
-    CONTROL = 4
-    TRAIN = 5
-    # TODO make Dump a control request to be passed to the Pod directly
-    DUMP = 6
+    DATA = 0
+    CONTROL = 1
 
 
 class CompressAlgo(BetterEnum):
@@ -308,9 +228,8 @@ class OnErrorStrategy(BetterEnum):
     IGNORE = (
         0  #: Ignore it, keep running all Drivers & Executors logics in the sequel flow
     )
-    SKIP_EXECUTOR = 1  #: Skip all Executors in the sequel, but drivers are still called
-    SKIP_HANDLE = 2  #: Skip all Drivers & Executors in the sequel, only `pre_hook` and `post_hook` are called
-    THROW_EARLY = 3  #: Immediately throw the exception, the sequel flow will not be running at all
+    SKIP_HANDLE = 1  #: Skip all Executors in the sequel, only `pre_hook` and `post_hook` are called
+    THROW_EARLY = 2  #: Immediately throw the exception, the sequel flow will not be running at all
 
 
 class FlowInspectType(BetterEnum):
@@ -318,7 +237,7 @@ class FlowInspectType(BetterEnum):
 
     HANG = 0  # keep them hanging there
     REMOVE = 1  # remove them in the build
-    COLLECT = 2  # spawn a new pod and collect them before build
+    COLLECT = 2  # spawn a new deployment and collect them before build
 
     @property
     def is_keep(self) -> bool:
@@ -330,29 +249,13 @@ class FlowInspectType(BetterEnum):
         return self.value in {0, 2}
 
 
-class RemoteAccessType(BetterEnum):
-    """Remote access type when connect to the host."""
-
-    SSH = 0  # ssh connection
-    JINAD = 1  # using rest api via jinad
-
-
-class BuildTestLevel(BetterEnum):
-    """Test level in :command:`jina hub build`, higher level includes lower levels."""
-
-    NONE = 0  # no build test
-    EXECUTOR = 1  # test at executor level, directly use the config yaml
-    POD_NONDOCKER = 2  # test at pod level, directly use the config yaml
-    POD_DOCKER = 3  # test at pod level but pod --uses the built image
-    FLOW = 4  # test at a simple flow
-
-
 class DataInputType(BetterEnum):
     """Data input type in the request generator."""
 
     AUTO = 0  # auto inference the input type from data (!WARN: could be slow as it relies on try-execept)
     DOCUMENT = 1  # the input is a full document
     CONTENT = 2  # the input is just the content of the document
+    DICT = 3  # the input is a dictionary representing a Document, needed while pydantic model not available
 
 
 class RuntimeBackendType(BetterEnum):
@@ -362,80 +265,15 @@ class RuntimeBackendType(BetterEnum):
     PROCESS = 1
 
 
-class EmbeddingClsType(BetterEnum):
-    """Enums for representing the type of embeddings supported."""
+class RemoteWorkspaceState(str, Enum):
+    """Enum representing state of remote workspace created by JinaD"""
 
-    DENSE = 0
-    SCIPY_COO = 1
-    SCIPY_CSR = 2
-    SCIPY_BSR = 3
-    SCIPY_CSC = 4
-    TORCH = 5
-    TF = 6
-
-    @property
-    def is_sparse(self) -> bool:
-        """
-        Check if is of sparse type
-
-        :return: True if the type is sparse
-        """
-        return self.value != 0
-
-    @property
-    def is_dense(self) -> bool:
-        """
-        Check if is of dense type
-
-        :return: True if the type is dense
-        """
-        return self.value == 0
-
-    @property
-    def is_scipy(self) -> bool:
-        """
-        Check if is of scipy sparse type
-
-        :return: True is of scipy sparse type
-        """
-        return self.value in [1, 2, 3, 4]
-
-    @property
-    def is_torch(self) -> bool:
-        """
-        Check if is of torch sparse type
-
-        :return: True is of torch sparse type
-        """
-        return self.value == 5
-
-    @property
-    def is_tf(self) -> bool:
-        """
-        Check if is of tf sparse type
-
-        :return: True is of tf sparse type
-        """
-        return self.value == 6
-
-    @property
-    def scipy_cls_type(self) -> str:
-        """
-        Return the specific scipy class type (coo, csr, csc, bsr)
-
-        :return: True is of scipy sparse type
-        """
-        if self.is_scipy:
-            return self.name.split('_')[1].lower()
-
-    @property
-    def is_scipy_stackable(self) -> bool:
-        """
-        Return if the specific scipy class is stackable. (BSR and CSC when stacked are converted into COO)
-
-        :return: True is class is stackable
-        """
-        return self.value in [1, 2]
+    PENDING = 'PENDING'
+    CREATING = 'CREATING'
+    UPDATING = 'UPDATING'
+    ACTIVE = 'ACTIVE'
+    FAILED = 'FAILED'
+    DELETING = 'DELETING'
 
 
 def replace_enum_to_str(obj):
