@@ -10,53 +10,60 @@ import sys
 import threading
 import time
 import uuid
+import warnings
 from collections import OrderedDict
 from contextlib import ExitStack
 from typing import (
-    Optional,
-    Union,
-    Tuple,
-    List,
-    Set,
-    Dict,
-    overload,
-    Type,
     TYPE_CHECKING,
+    Dict,
+    List,
+    Optional,
+    Set,
+    Tuple,
+    Type,
+    Union,
+    overload,
 )
 
-from jina.orchestrate.flow.builder import allowed_levels, _hanging_deployments
+from rich import print
+from rich.table import Table
+
 from jina import __default_host__, helper
 from jina.clients import Client
 from jina.clients.mixin import AsyncPostMixin, PostMixin
 from jina.enums import (
-    FlowBuildLevel,
     DeploymentRoleType,
+    FlowBuildLevel,
     FlowInspectType,
     GatewayProtocolType,
 )
 from jina.excepts import (
-    FlowTopologyError,
     FlowMissingDeploymentError,
+    FlowTopologyError,
     RuntimeFailToStart,
 )
 from jina.helper import (
-    colored,
-    get_public_ip,
-    get_internal_ip,
-    typename,
+    GRAPHQL_MIN_DOCARRAY_VERSION,
     ArgNamespace,
-    download_mermaid_url,
     CatchAllCleanupContextManager,
+    colored_rich,
+    docarray_graphql_compatible,
+    download_mermaid_url,
+    get_internal_ip,
+    get_public_ip,
+    get_rich_console,
+    typename,
 )
 from jina.jaml import JAMLCompatible
 from jina.logging.logger import JinaLogger
+from jina.orchestrate.deployments import Deployment
+from jina.orchestrate.flow.builder import _hanging_deployments, allowed_levels
 from jina.parsers import (
-    set_gateway_parser,
-    set_deployment_parser,
     set_client_cli_parser,
+    set_deployment_parser,
+    set_gateway_parser,
 )
 from jina.parsers.flow import set_flow_parser
-from jina.orchestrate.deployments import Deployment
 
 __all__ = ['Flow']
 
@@ -70,9 +77,9 @@ class FlowType(type(ExitStack), type(JAMLCompatible)):
 _regex_port = r'(.*?):([0-9]{1,4}|[1-5][0-9]{4}|6[0-4][0-9]{3}|65[0-4][0-9]{2}|655[0-2][0-9]|6553[0-5])$'
 
 if TYPE_CHECKING:
-    from jina.serve.executors import BaseExecutor
     from jina.clients.base import BaseClient
     from jina.orchestrate.flow.asyncio import AsyncFlow
+    from jina.serve.executors import BaseExecutor
 
 GATEWAY_NAME = 'gateway'
 FALLBACK_PARSERS = [
@@ -84,7 +91,7 @@ FALLBACK_PARSERS = [
 
 
 class Flow(PostMixin, JAMLCompatible, ExitStack, metaclass=FlowType):
-    """Flow is how Jina streamlines and distributes Executors. """
+    """Flow is how Jina streamlines and distributes Executors."""
 
     # overload_inject_start_client_flow
     @overload
@@ -97,7 +104,7 @@ class Flow(PostMixin, JAMLCompatible, ExitStack, metaclass=FlowType):
         port: Optional[int] = None,
         protocol: Optional[str] = 'GRPC',
         proxy: Optional[bool] = False,
-        results_as_docarray: Optional[bool] = False,
+        return_responses: Optional[bool] = False,
         **kwargs,
     ):
         """Create a Flow. Flow is how Jina streamlines and scales Executors. This overloaded method provides arguments from `jina client` CLI.
@@ -108,7 +115,7 @@ class Flow(PostMixin, JAMLCompatible, ExitStack, metaclass=FlowType):
         :param port: The port of the Gateway, which the client should connect to.
         :param protocol: Communication protocol between server and client.
         :param proxy: If set, respect the http_proxy and https_proxy environment variables. otherwise, it will unset these proxy variables before start. gRPC seems to prefer no proxy
-        :param results_as_docarray: If set, return results as DocArray instead of Request.
+        :param return_responses: If set, return results as List of Requests instead of a reduced DocArray.
 
         .. # noqa: DAR202
         .. # noqa: DAR101
@@ -122,18 +129,16 @@ class Flow(PostMixin, JAMLCompatible, ExitStack, metaclass=FlowType):
     def __init__(
         self,
         *,
-        compress: Optional[str] = 'NONE',
-        compress_min_bytes: Optional[int] = 1024,
-        compress_min_ratio: Optional[float] = 1.1,
         connection_list: Optional[str] = None,
         cors: Optional[bool] = False,
-        daemon: Optional[bool] = False,
         default_swagger_ui: Optional[bool] = False,
         deployments_addresses: Optional[str] = '{}',
         description: Optional[str] = None,
+        disable_reduce: Optional[bool] = False,
         env: Optional[dict] = None,
         expose_endpoints: Optional[str] = None,
-        expose_public: Optional[bool] = False,
+        expose_graphql_endpoint: Optional[bool] = False,
+        graph_conditions: Optional[str] = '{}',
         graph_description: Optional[str] = '{}',
         host: Optional[str] = '0.0.0.0',
         host_in: Optional[str] = '0.0.0.0',
@@ -142,9 +147,9 @@ class Flow(PostMixin, JAMLCompatible, ExitStack, metaclass=FlowType):
         native: Optional[bool] = False,
         no_crud_endpoints: Optional[bool] = False,
         no_debug_endpoints: Optional[bool] = False,
+        output_array_type: Optional[str] = None,
         polling: Optional[str] = 'ANY',
-        port_expose: Optional[int] = None,
-        port_in: Optional[int] = None,
+        port: Optional[int] = None,
         prefetch: Optional[int] = 0,
         protocol: Optional[str] = 'GRPC',
         proxy: Optional[bool] = False,
@@ -170,21 +175,16 @@ class Flow(PostMixin, JAMLCompatible, ExitStack, metaclass=FlowType):
     ):
         """Create a Flow. Flow is how Jina streamlines and scales Executors. This overloaded method provides arguments from `jina gateway` CLI.
 
-        :param compress: The compress algorithm used over the entire Flow.
-
-              Note that this is not necessarily effective,
-              it depends on the settings of `--compress-min-bytes` and `compress-min-ratio`
-        :param compress_min_bytes: The original message size must be larger than this number to trigger the compress algorithm, -1 means disable compression.
-        :param compress_min_ratio: The compression ratio (uncompressed_size/compressed_size) must be higher than this number to trigger the compress algorithm.
         :param connection_list: dictionary JSON with a list of connections to configure
         :param cors: If set, a CORS middleware is added to FastAPI frontend to allow cross-origin access.
-        :param daemon: The Pod attempts to terminate all of its Runtime child processes/threads on existing. setting it to true basically tell the Pod do not wait on the Runtime when closing
         :param default_swagger_ui: If set, the default swagger ui is used for `/docs` endpoint.
         :param deployments_addresses: dictionary JSON with the input addresses of each Deployment
         :param description: The description of this HTTP server. It will be used in automatics docs such as Swagger UI.
+        :param disable_reduce: Disable the built-in reduce mechanism, set this if the reduction is to be handled by the Executor connected to this Head
         :param env: The map of environment variables that are available inside runtime
         :param expose_endpoints: A JSON string that represents a map from executor endpoints (`@requests(on=...)`) to HTTP endpoints.
-        :param expose_public: If set, expose the public IP address to remote when necessary, by default it exposesprivate IP address, which only allows accessing under the same network/subnet. Important to set this to true when the Pod will receive input connections from remote Pods
+        :param expose_graphql_endpoint: If set, /graphql endpoint is added to HTTP interface.
+        :param graph_conditions: Dictionary stating which filtering conditions each Executor in the graph requires to receive Documents.
         :param graph_description: Routing graph for the gateway
         :param host: The host address of the runtime, by default it is 0.0.0.0.
         :param host_in: The host address for binding to, by default it is 0.0.0.0
@@ -203,6 +203,11 @@ class Flow(PostMixin, JAMLCompatible, ExitStack, metaclass=FlowType):
 
                   Any executor that has `@requests(on=...)` bind with those values will receive data requests.
         :param no_debug_endpoints: If set, /status /post endpoints are removed from HTTP interface.
+        :param output_array_type: The type of array `tensor` and `embedding` will be serialized to.
+
+          Supports the same types as `docarray.to_protobuf(.., ndarray_type=...)`, which can be found
+          `here <https://docarray.jina.ai/fundamentals/document/serialization/#from-to-protobuf>`.
+          Defaults to retaining whatever type is returned by the Executor.
         :param polling: The polling strategy of the Deployment and its endpoints (when `shards>1`).
               Can be defined for all endpoints of a Deployment or by endpoint.
               Define per Deployment:
@@ -211,8 +216,7 @@ class Flow(PostMixin, JAMLCompatible, ExitStack, metaclass=FlowType):
               Define per Endpoint:
               JSON dict, {endpoint: PollingType}
               {'/custom': 'ALL', '/search': 'ANY', '*': 'ANY'}
-        :param port_expose: The port that the gateway exposes for clients for GRPC connections.
-        :param port_in: The port for input data to bind to, default a random port between [49152, 65535]
+        :param port: The port for input data to bind to, default is a random port between [49152, 65535]
         :param prefetch: Number of requests fetched from the client before feeding into the first Executor.
 
               Used to control the speed of data input into a Flow. 0 disables prefetch (disabled by default)
@@ -229,7 +233,7 @@ class Flow(PostMixin, JAMLCompatible, ExitStack, metaclass=FlowType):
         :param replicas: The number of replicas in the deployment
         :param runtime_backend: The parallel backend of the runtime inside the Pod
         :param runtime_cls: The runtime class to run inside the Pod
-        :param shards: The number of shards in the deployment running at the same time. For more details check https://docs.jina.ai/fundamentals/flow/topology/
+        :param shards: The number of shards in the deployment running at the same time. For more details check https://docs.jina.ai/fundamentals/flow/create-flow/#complex-flow-topologies
         :param timeout_ctrl: The timeout in milliseconds of the control request, -1 for waiting forever
         :param timeout_ready: The timeout in milliseconds of a Pod waits for the runtime to be ready, -1 for waiting forever
         :param title: The title of this HTTP server. It will be used in automatics docs such as Swagger UI.
@@ -265,6 +269,7 @@ class Flow(PostMixin, JAMLCompatible, ExitStack, metaclass=FlowType):
         self,
         *,
         env: Optional[dict] = None,
+        expose_graphql_endpoint: Optional[bool] = False,
         inspect: Optional[str] = 'COLLECT',
         log_config: Optional[str] = None,
         name: Optional[str] = None,
@@ -279,6 +284,7 @@ class Flow(PostMixin, JAMLCompatible, ExitStack, metaclass=FlowType):
         """Create a Flow. Flow is how Jina streamlines and scales Executors. This overloaded method provides arguments from `jina flow` CLI.
 
         :param env: The map of environment variables that are available inside runtime
+        :param expose_graphql_endpoint: If set, /graphql endpoint is added to HTTP interface.
         :param inspect: The strategy on those inspect deployments in the flow.
 
               If `REMOVE` is given then all inspect deployments are removed when building the flow.
@@ -327,6 +333,17 @@ class Flow(PostMixin, JAMLCompatible, ExitStack, metaclass=FlowType):
             GATEWAY_NAME
         ]  #: default first deployment is gateway, will add when build()
         self._update_args(args, **kwargs)
+        if (
+            self.protocol == GatewayProtocolType.HTTP
+            and self.args.expose_graphql_endpoint
+            and not docarray_graphql_compatible()
+        ):
+            self.args.expose_graphql_endpoint = False
+            warnings.warn(
+                'DocArray version is incompatible with GraphQL features. '
+                'Automatically setting expose_graphql_endpoint=False. '
+                f'To use GraphQL features, install docarray>={GRAPHQL_MIN_DOCARRAY_VERSION}'
+            )
 
         if isinstance(self.args, argparse.Namespace):
             self.logger = JinaLogger(
@@ -336,8 +353,8 @@ class Flow(PostMixin, JAMLCompatible, ExitStack, metaclass=FlowType):
             self.logger = JinaLogger(self.__class__.__name__, **self._common_kwargs)
 
     def _update_args(self, args, **kwargs):
-        from jina.parsers.flow import set_flow_parser
         from jina.helper import ArgNamespace
+        from jina.parsers.flow import set_flow_parser
 
         _flow_parser = set_flow_parser()
         if args is None:
@@ -425,6 +442,7 @@ class Flow(PostMixin, JAMLCompatible, ExitStack, metaclass=FlowType):
         needs: str,
         graph_description: Dict[str, List[str]],
         deployments_addresses: Dict[str, List[str]],
+        graph_conditions: Dict[str, Dict],
         **kwargs,
     ):
         kwargs.update(
@@ -433,7 +451,7 @@ class Flow(PostMixin, JAMLCompatible, ExitStack, metaclass=FlowType):
                 ctrl_with_ipc=True,  # otherwise ctrl port would be conflicted
                 host=self.host,
                 protocol=self.protocol,
-                port_expose=self.port_expose,
+                port=self.port,
                 deployment_role=DeploymentRoleType.GATEWAY,
                 expose_endpoints=json.dumps(self._endpoints_mapping),
             )
@@ -442,7 +460,11 @@ class Flow(PostMixin, JAMLCompatible, ExitStack, metaclass=FlowType):
         kwargs.update(self._common_kwargs)
         args = ArgNamespace.kwargs2namespace(kwargs, set_gateway_parser())
         args.noblock_on_start = True
+        args.expose_graphql_endpoint = (
+            self.args.expose_graphql_endpoint
+        )  # also used in Flow, thus not in kwargs
         args.graph_description = json.dumps(graph_description)
+        args.graph_conditions = json.dumps(graph_conditions)
         args.deployments_addresses = json.dumps(deployments_addresses)
         self._deployment_nodes[GATEWAY_NAME] = Deployment(args, needs)
 
@@ -451,7 +473,7 @@ class Flow(PostMixin, JAMLCompatible, ExitStack, metaclass=FlowType):
         for node, v in self._deployment_nodes.items():
             if node == 'gateway':
                 continue
-            graph_dict[node] = [f'{v.protocol}://{v.host}:{v.head_port_in}']
+            graph_dict[node] = [f'{v.protocol}://{v.host}:{v.head_port}']
 
         return graph_dict
 
@@ -459,8 +481,8 @@ class Flow(PostMixin, JAMLCompatible, ExitStack, metaclass=FlowType):
         self, k8s_namespace: str, k8s_connection_pool: bool
     ) -> Dict[str, List[str]]:
         graph_dict = {}
-        from jina.serve.networking import K8sGrpcConnectionPool
         from jina.orchestrate.deployments.config.helper import to_compatible_name
+        from jina.serve.networking import K8sGrpcConnectionPool
 
         for node, v in self._deployment_nodes.items():
             if node == 'gateway':
@@ -476,26 +498,34 @@ class Flow(PostMixin, JAMLCompatible, ExitStack, metaclass=FlowType):
             # we only need hard coded addresses if the k8s connection pool is disabled or if this deployment is external
             if not k8s_connection_pool or v.external:
                 graph_dict[node] = [
-                    f'{deployment_k8s_address}:{v.head_port_in if v.external else K8sGrpcConnectionPool.K8S_PORT_IN}'
+                    f'{deployment_k8s_address}:{v.head_port if v.external else K8sGrpcConnectionPool.K8S_PORT}'
                 ]
 
         return graph_dict if graph_dict else None
 
     def _get_docker_compose_deployments_addresses(self) -> Dict[str, List[str]]:
         graph_dict = {}
-        from jina.orchestrate.deployments.config.docker_compose import PORT_IN
+        from jina.orchestrate.deployments.config.docker_compose import port
         from jina.orchestrate.deployments.config.helper import to_compatible_name
 
         for node, v in self._deployment_nodes.items():
             if node == 'gateway':
                 continue
             deployment_docker_compose_address = (
-                f'{to_compatible_name(v.head_args.name)}:{PORT_IN}'
+                f'{to_compatible_name(v.head_args.name)}:{port}'
             )
             graph_dict[node] = [deployment_docker_compose_address]
 
         return graph_dict
 
+    def _get_graph_conditions(self) -> Dict[str, Dict]:
+        graph_condition = {}
+        for node, v in self._deployment_nodes.items():
+            if v.args.when is not None:  # condition on input docs
+                graph_condition[node] = v.args.when
+
+        return graph_condition
+
     def _get_graph_representation(self) -> Dict[str, List[str]]:
         def _add_node(graph, n):
             # in the graph we need to distinguish between start and end gateway, although they are the same deployment
@@ -569,11 +599,10 @@ class Flow(PostMixin, JAMLCompatible, ExitStack, metaclass=FlowType):
         self,
         *,
         connection_list: Optional[str] = None,
-        daemon: Optional[bool] = False,
+        disable_reduce: Optional[bool] = False,
         docker_kwargs: Optional[dict] = None,
         entrypoint: Optional[str] = None,
         env: Optional[dict] = None,
-        expose_public: Optional[bool] = False,
         external: Optional[bool] = False,
         force_update: Optional[bool] = False,
         gpus: Optional[str] = None,
@@ -583,8 +612,9 @@ class Flow(PostMixin, JAMLCompatible, ExitStack, metaclass=FlowType):
         log_config: Optional[str] = None,
         name: Optional[str] = None,
         native: Optional[bool] = False,
+        output_array_type: Optional[str] = None,
         polling: Optional[str] = 'ANY',
-        port_in: Optional[int] = None,
+        port: Optional[int] = None,
         port_jinad: Optional[int] = 8000,
         pull_latest: Optional[bool] = False,
         py_modules: Optional[List[str]] = None,
@@ -607,20 +637,20 @@ class Flow(PostMixin, JAMLCompatible, ExitStack, metaclass=FlowType):
         uses_requests: Optional[dict] = None,
         uses_with: Optional[dict] = None,
         volumes: Optional[List[str]] = None,
+        when: Optional[dict] = None,
         workspace: Optional[str] = None,
         **kwargs,
     ) -> Union['Flow', 'AsyncFlow']:
         """Add an Executor to the current Flow object.
 
         :param connection_list: dictionary JSON with a list of connections to configure
-        :param daemon: The Pod attempts to terminate all of its Runtime child processes/threads on existing. setting it to true basically tell the Pod do not wait on the Runtime when closing
+        :param disable_reduce: Disable the built-in reduce mechanism, set this if the reduction is to be handled by the Executor connected to this Head
         :param docker_kwargs: Dictionary of kwargs arguments that will be passed to Docker SDK when starting the docker '
           container.
 
           More details can be found in the Docker SDK docs:  https://docker-py.readthedocs.io/en/stable/
         :param entrypoint: The entrypoint command overrides the ENTRYPOINT in Docker image. when not set then the Docker image ENTRYPOINT takes effective.
         :param env: The map of environment variables that are available inside runtime
-        :param expose_public: If set, expose the public IP address to remote when necessary, by default it exposesprivate IP address, which only allows accessing under the same network/subnet. Important to set this to true when the Pod will receive input connections from remote Pods
         :param external: The Deployment will be considered an external Deployment that has been started independently from the Flow.This Deployment will not be context managed by the Flow.
         :param force_update: If set, always pull the latest Hub Executor bundle even it exists on local
         :param gpus: This argument allows dockerized Jina executor discover local gpu devices.
@@ -645,6 +675,11 @@ class Flow(PostMixin, JAMLCompatible, ExitStack, metaclass=FlowType):
 
           When not given, then the default naming strategy will apply.
         :param native: If set, only native Executors is allowed, and the Executor is always run inside WorkerRuntime.
+        :param output_array_type: The type of array `tensor` and `embedding` will be serialized to.
+
+          Supports the same types as `docarray.to_protobuf(.., ndarray_type=...)`, which can be found
+          `here <https://docarray.jina.ai/fundamentals/document/serialization/#from-to-protobuf>`.
+          Defaults to retaining whatever type is returned by the Executor.
         :param polling: The polling strategy of the Deployment and its endpoints (when `shards>1`).
               Can be defined for all endpoints of a Deployment or by endpoint.
               Define per Deployment:
@@ -653,7 +688,7 @@ class Flow(PostMixin, JAMLCompatible, ExitStack, metaclass=FlowType):
               Define per Endpoint:
               JSON dict, {endpoint: PollingType}
               {'/custom': 'ALL', '/search': 'ANY', '*': 'ANY'}
-        :param port_in: The port for input data to bind to, default a random port between [49152, 65535]
+        :param port: The port for input data to bind to, default is a random port between [49152, 65535]
         :param port_jinad: The port of the remote machine for usage with JinaD.
         :param pull_latest: Pull the latest image before running
         :param py_modules: The customized python modules need to be imported before loading the executor
@@ -668,7 +703,7 @@ class Flow(PostMixin, JAMLCompatible, ExitStack, metaclass=FlowType):
         :param replicas: The number of replicas in the deployment
         :param runtime_backend: The parallel backend of the runtime inside the Pod
         :param runtime_cls: The runtime class to run inside the Pod
-        :param shards: The number of shards in the deployment running at the same time. For more details check https://docs.jina.ai/fundamentals/flow/topology/
+        :param shards: The number of shards in the deployment running at the same time. For more details check https://docs.jina.ai/fundamentals/flow/create-flow/#complex-flow-topologies
         :param timeout_ctrl: The timeout in milliseconds of the control request, -1 for waiting forever
         :param timeout_ready: The timeout in milliseconds of a Pod waits for the runtime to be ready, -1 for waiting forever
         :param upload_files: The files on the host to be uploaded to the remote
@@ -703,6 +738,7 @@ class Flow(PostMixin, JAMLCompatible, ExitStack, metaclass=FlowType):
           - If separated by `:`, then the first part will be considered as the local host path and the second part is the path in the container system.
           - If no split provided, then the basename of that directory will be mounted into container's root path, e.g. `--volumes="/user/test/my-workspace"` will be mounted into `/my-workspace` inside the container.
           - All volumes are mounted with read-write mode.
+        :param when: The condition that the documents need to fulfill before reaching the Executor.The condition can be defined in the form of a `DocArray query condition <https://docarray.jina.ai/fundamentals/documentarray/find/#query-by-conditions>`
         :param workspace: The working directory for any IO operations in this object. If not set, then derive from its parent `workspace`.
         :return: a (new) Flow object with modification
 
@@ -761,7 +797,8 @@ class Flow(PostMixin, JAMLCompatible, ExitStack, metaclass=FlowType):
 
         # set the kwargs inherit from `Flow(kwargs1=..., kwargs2=)`
         for key, value in op_flow._common_kwargs.items():
-            if key not in kwargs:
+            # do not inherit the port argument from the flow
+            if key not in kwargs and key != 'port':
                 kwargs[key] = value
 
         # check if host is set to remote:port
@@ -800,10 +837,10 @@ class Flow(PostMixin, JAMLCompatible, ExitStack, metaclass=FlowType):
         args.noblock_on_start = True
         args.extra_search_paths = self.args.extra_search_paths
 
-        port_in = kwargs.get('port_in', None)
-        if not port_in:
-            port_in = helper.random_port()
-            args.port_in = port_in
+        port = kwargs.get('port', None)
+        if not port:
+            port = helper.random_port()
+            args.port = port
 
         op_flow._deployment_nodes[deployment_name] = Deployment(args, needs)
 
@@ -918,7 +955,7 @@ class Flow(PostMixin, JAMLCompatible, ExitStack, metaclass=FlowType):
             f'{prefix}-{GATEWAY_NAME}',
             {
                 'host': gateway_deployment.head_host,
-                'port': gateway_deployment.head_port_in,
+                'port': gateway_deployment.head_port,
                 'expected_parts': 0,
             },
         )
@@ -963,6 +1000,7 @@ class Flow(PostMixin, JAMLCompatible, ExitStack, metaclass=FlowType):
                 needs={op_flow.last_deployment},
                 graph_description=op_flow._get_graph_representation(),
                 deployments_addresses=op_flow._get_deployments_addresses(),
+                graph_conditions=op_flow._get_graph_conditions(),
             )
 
         removed_deployments = []
@@ -1099,10 +1137,7 @@ class Flow(PostMixin, JAMLCompatible, ExitStack, metaclass=FlowType):
             except Exception as ex:
                 results[_deployment_name] = repr(ex)
 
-        def _polling_status():
-            spinner = itertools.cycle(
-                ['⠋', '⠙', '⠹', '⠸', '⠼', '⠴', '⠦', '⠧', '⠇', '⠏']
-            )
+        def _polling_status(status):
 
             while True:
                 num_all = len(results)
@@ -1114,15 +1149,13 @@ class Flow(PostMixin, JAMLCompatible, ExitStack, metaclass=FlowType):
                         pendings.append(_k)
                     else:
                         num_done += 1
-                sys.stdout.write('\r{}\r'.format(' ' * 100))
-                pending_str = colored(' '.join(pendings)[:50], 'yellow')
-                sys.stdout.write(
-                    f'{colored(next(spinner), "green")} {num_done}/{num_all} waiting {pending_str} to be ready...'
+                pending_str = colored_rich(' '.join(pendings)[:50], 'yellow')
+
+                status.update(
+                    f'{num_done}/{num_all} waiting {pending_str} to be ready...'
                 )
-                sys.stdout.flush()
 
                 if not pendings:
-                    sys.stdout.write('\r{}\r'.format(' ' * 100))
                     break
                 time.sleep(0.1)
 
@@ -1139,38 +1172,43 @@ class Flow(PostMixin, JAMLCompatible, ExitStack, metaclass=FlowType):
             threads.append(t)
             t.start()
 
-        # kick off spinner thread
-        t_m = threading.Thread(target=_polling_status, daemon=True)
-        t_m.start()
+        console = get_rich_console()
+        with console.status('Working...') as status:
+            # kick off spinner thread
+            t_m = threading.Thread(target=_polling_status, args=[status], daemon=True)
+            t_m.start()
 
-        # kick off ip getter thread
-        addr_table = []
-        t_ip = threading.Thread(
-            target=self._get_address_table, args=(addr_table,), daemon=True
+            # kick off ip getter thread
+            addr_table = self._init_table()
+
+            t_ip = threading.Thread(
+                target=self._get_address_table, args=(addr_table,), daemon=True
+            )
+            t_ip.start()
+
+            for t in threads:
+                t.join()
+            if t_ip is not None:
+                t_ip.join()
+            t_m.join()
+
+            error_deployments = [k for k, v in results.items() if v != 'done']
+            if error_deployments:
+                self.logger.error(
+                    f'Flow is aborted due to {error_deployments} can not be started.'
+                )
+                self.close()
+                raise RuntimeFailToStart
+
+        success_msg = '[green]🎉 Flow is ready to use![/green]'
+        console.print(success_msg)
+        if addr_table:
+            print(
+                addr_table
+            )  # can't use logger here see : https://github.com/Textualize/rich/discussions/2024
+        self.logger.debug(
+            f'{self.num_deployments} Deployments (i.e. {self.num_pods} Pods) are running in this Flow'
         )
-        t_ip.start()
-
-        for t in threads:
-            t.join()
-        if t_ip is not None:
-            t_ip.join()
-        t_m.join()
-
-        error_deployments = [k for k, v in results.items() if v != 'done']
-        if error_deployments:
-            self.logger.error(
-                f'Flow is aborted due to {error_deployments} can not be started.'
-            )
-            self.close()
-            raise RuntimeFailToStart
-        else:
-            success_msg = colored('🎉 Flow is ready to use!', 'green')
-
-            if addr_table:
-                self.logger.info(success_msg + '\n' + '\n'.join(addr_table))
-            self.logger.debug(
-                f'{self.num_deployments} Deployments (i.e. {self.num_pods} Pods) are running in this Flow'
-            )
 
     @property
     def num_deployments(self) -> int:
@@ -1219,9 +1257,8 @@ class Flow(PostMixin, JAMLCompatible, ExitStack, metaclass=FlowType):
 
         kwargs = dict(
             host=self.host,
-            port=self.port_expose,
+            port=self.port,
             protocol=self.protocol,
-            results_as_docarray=True,
         )
         kwargs.update(self._common_kwargs)
         return Client(**kwargs)
@@ -1350,7 +1387,7 @@ class Flow(PostMixin, JAMLCompatible, ExitStack, metaclass=FlowType):
         showed = False
         if inline_display:
             try:
-                from IPython.display import display, Image
+                from IPython.display import Image, display
 
                 display(Image(url=url))
                 showed = True
@@ -1384,26 +1421,26 @@ class Flow(PostMixin, JAMLCompatible, ExitStack, metaclass=FlowType):
         return f'https://mermaid.ink/{img_type}/{encoded_str}'
 
     @property
-    def port_expose(self) -> int:
+    def port(self) -> int:
         """Return the exposed port of the gateway
         .. # noqa: DAR201
         """
         if GATEWAY_NAME in self._deployment_nodes:
-            return self._deployment_nodes[GATEWAY_NAME].args.port_expose
+            return self._deployment_nodes[GATEWAY_NAME].port
         else:
-            return self._common_kwargs.get('port_expose', None)
+            return self._common_kwargs.get('port', None)
 
-    @port_expose.setter
-    def port_expose(self, value: int):
+    @port.setter
+    def port(self, value: int):
         """Set the new exposed port of the Flow (affects Gateway and Client)
 
         :param value: the new port to expose
         """
-        self._common_kwargs['port_expose'] = value
+        self._common_kwargs['port'] = value
 
         # Flow is build to graph already
         if self._build_level >= FlowBuildLevel.GRAPH:
-            self[GATEWAY_NAME].args.port_expose = self._common_kwargs['port_expose']
+            self[GATEWAY_NAME].args.port = self._common_kwargs['port']
 
         # Flow is running already, then close the existing gateway
         if self._build_level >= FlowBuildLevel.RUNNING:
@@ -1458,46 +1495,54 @@ class Flow(PostMixin, JAMLCompatible, ExitStack, metaclass=FlowType):
     def __iter__(self):
         return self._deployment_nodes.items().__iter__()
 
+    def _init_table(self):
+        table = Table(title=None, box=None, highlight=True)
+        table.add_column('', justify='right')
+        table.add_column('', justify='right')
+        table.add_column('', justify='right')
+        table.add_column('', justify='right')
+
+        return table
+
     def _get_address_table(self, address_table):
-        address_table.extend(
-            [
-                f'\t🔗 Protocol: \t\t{colored(self.protocol, attrs="bold")}',
-                f'\t🏠 Local access:\t'
-                + colored(f'{self.host}:{self.port_expose}', 'cyan', attrs='underline'),
-                f'\t🔒 Private network:\t'
-                + colored(
-                    f'{self.address_private}:{self.port_expose}',
-                    'cyan',
-                    attrs='underline',
-                ),
-            ]
+        address_table.add_row('🔗', 'Protocol: ', f'{self.protocol}')
+        address_table.add_row(
+            '🏠',
+            'Local access: ',
+            f'[underline]{self.host}:{self.port}[/underline]',
         )
+        address_table.add_row(
+            '🔒',
+            'Private network: ',
+            f'[underline]{self.address_private}:{self.port}[/underline]',
+        )
+
         if self.address_public:
-            address_table.append(
-                f'\t🌐 Public address:\t'
-                + colored(
-                    f'{self.address_public}:{self.port_expose}',
-                    'cyan',
-                    attrs='underline',
-                )
+            address_table.add_row(
+                '🌐',
+                'Public address: ',
+                f'[underline]{self.address_public}:{self.port}[/underline]',
             )
+
         if self.protocol == GatewayProtocolType.HTTP:
-            address_table.append(
-                f'\t💬 Swagger UI:\t\t'
-                + colored(
-                    f'http://localhost:{self.port_expose}/docs',
-                    'cyan',
-                    attrs='underline',
-                )
+            address_table.add_row(
+                '💬',
+                'Swagger UI: ',
+                f'[underline]http://localhost:{self.port}/docs[/underline]',
             )
-            address_table.append(
-                f'\t📚 Redoc:\t\t'
-                + colored(
-                    f'http://localhost:{self.port_expose}/redoc',
-                    'cyan',
-                    attrs='underline',
-                )
+
+            address_table.add_row(
+                '📚',
+                'Redoc: ',
+                f'[underline]http://localhost:{self.port}/redoc[/underline]',
             )
+            if self.args.expose_graphql_endpoint:
+                address_table.add_row(
+                    '💬',
+                    'GraphQL UI: ',
+                    f'[underline][cyan]http://localhost:{self.port}/graphql[/underline][/cyan]',
+                )
+
         return address_table
 
     def block(
@@ -1842,9 +1887,9 @@ class Flow(PostMixin, JAMLCompatible, ExitStack, metaclass=FlowType):
 
         # noqa: DAR201
         """
-        if 'port_expose' in self._common_kwargs:
+        if 'port' in self._common_kwargs:
             kwargs = copy.deepcopy(self._common_kwargs)
-            kwargs['port'] = self._common_kwargs['port_expose']
+            kwargs['port'] = self._common_kwargs['port']
 
         return ArgNamespace.kwargs2namespace(kwargs, set_client_cli_parser())
 
